@@ -3,7 +3,7 @@ package CGI::Ex::Validate;
 ### CGI Extended Validator
 
 ###----------------------------------------------------------------###
-#  Copyright 2003 - Paul Seamons                                     #
+#  Copyright 2004 - Paul Seamons                                     #
 #  Distributed under the Perl Artistic License without warranty      #
 ###----------------------------------------------------------------###
 
@@ -21,7 +21,7 @@ use vars qw($VERSION
             @UNSUPPORTED_BROWSERS
             );
 
-$VERSION = '1.13';
+$VERSION = '1.14';
 
 $ERROR_PACKAGE = 'CGI::Ex::Validate::Error';
 $DEFAULT_EXT   = 'val';
@@ -150,7 +150,7 @@ sub validate {
     foreach (my $i = 0; $i <= $#$fields; $i ++) {
       my $ref = $fields->[$i];
       if (! ref($ref) && $ref eq 'OR') {
-        $i += ($found) ? 2 : 1; # if found skip the OR altogether
+        $i ++ if $found; # if found skip the OR altogether
         $found = 1; # reset
         next;
       }
@@ -242,7 +242,7 @@ sub check_conditional {
     my $ref = $ifs->[$i];
     if (! ref $ref) {
       if ($ref eq 'OR') {
-        $i += ($found) ? 2 : 1; # if found skip the OR altogether
+        $i ++ if $found; # if found skip the OR altogether
         $found = 1; # reset
         next;
       } else {
@@ -256,7 +256,7 @@ sub check_conditional {
     last if ! $found;
 
     ### get the field - allow for custom variables based upon a match
-    my $field = $ref->{'field'} || die "Missing field key during validate_if";
+    my $field = $ref->{'field'} || die "Missing field key during validate_if (possibly used a reference to a main hash *foo -> &foo)";
     $field =~ s/\$(\d+)/defined($ifs_match->[$1]) ? $ifs_match->[$1] : ''/eg if $ifs_match;
 
     my @err = $self->validate_buddy($form, $field, $ref, $N_level);
@@ -280,7 +280,7 @@ sub validate_buddy {
   ### allow for not running some tests in the cgi
   if (scalar $self->filter_type('exclude_cgi',$types)) {
     delete $field_val->{'was_validated'};
-    return wantarray ? @errors : scalar @errors;
+    return wantarray ? @errors : $#errors + 1;
   }
 
   ### allow for field names that contain regular expressions
@@ -293,7 +293,7 @@ sub validate_buddy {
       my @match = (undef,$1,$2,$3,$4,$5); # limit to the matches
       push @errors, $self->validate_buddy($form, $_field, $field_val, $N_level, \@match);
     }
-    return wantarray ? @errors : scalar @errors;
+    return wantarray ? @errors : $#errors + 1;
   }
 
   ### allow for default value
@@ -362,9 +362,10 @@ sub validate_buddy {
   if ($modified) {
     if ($n_values == 1) {
       $form->{$field} = $values->[0];
-        $self->{cgi_object}->param(-name => $field, -value => $values->[0])
-          if $self->{cgi_object};
+      $self->{cgi_object}->param(-name => $field, -value => $values->[0])
+        if $self->{cgi_object};
     } else {
+      ### values in @{ $form->{$field} } were modified directly
       $self->{cgi_object}->param(-name => $field, -value => $values)
         if $self->{cgi_object};
     }
@@ -381,7 +382,7 @@ sub validate_buddy {
   }
   if (! $needs_val && $n_vif) {
     delete $field_val->{'was_validated'};
-    return wantarray ? @errors : scalar @errors;
+    return wantarray ? @errors : $#errors + 1;
   }
 
   ### check for simple existence
@@ -455,6 +456,8 @@ sub validate_buddy {
     }
   }
 
+  ### at this point @errors should still be empty
+  my $content_checked; # allow later for possible untainting (only happens if content was checked)
 
   ### loop on values of field
   foreach my $value (@$values) {
@@ -470,11 +473,13 @@ sub validate_buddy {
         return 1 if ! wantarray;
         $self->add_error(\@errors, $field, $type, $field_val, $ifs_match);
       }
+      $content_checked = 1;
     }
 
     ### field equality test
     foreach my $type ($self->filter_type('equals',$types)) {
       my $field2  = $field_val->{$type};
+      my $not     = ($field2 =~ s/^!\s*//) ? 1 : 0;
       my $success = 0;
       if ($field2 =~ m/^([\"\'])(.*)\1$/) {
         my $test = $2;
@@ -484,10 +489,11 @@ sub validate_buddy {
       } elsif (! defined($value)) {
         $success = 1; # occurs if they are both undefined
       }
-      if (! $success) {
+      if ($not ? $success : ! $success) {
         return 1 if ! wantarray;
         $self->add_error(\@errors, $field, $type, $field_val, $ifs_match);
       }
+      $content_checked = 1;
     }
 
     ### length min check
@@ -533,6 +539,7 @@ sub validate_buddy {
           }
         }
       }
+      $content_checked = 1;
     }
 
     ### allow for comparison checks
@@ -571,6 +578,7 @@ sub validate_buddy {
           $self->add_error(\@errors, $field, $type, $field_val, $ifs_match);
         }
       }
+      $content_checked = 1;
     }
 
     ### server side sql type
@@ -591,6 +599,7 @@ sub validate_buddy {
         return 1 if ! wantarray;
         $self->add_error(\@errors, $field, $type, $field_val, $ifs_match);
       }
+      $content_checked = 1;
     }
 
     ### server side custom type
@@ -599,6 +608,7 @@ sub validate_buddy {
       next if UNIVERSAL::isa($check, 'CODE') ? &$check($field, $value, $field_val, $type) : $check;
       return 1 if ! wantarray;
       $self->add_error(\@errors, $field, $type, $field_val, $ifs_match);
+      $content_checked = 1;
     }
 
     ### do specific type checks
@@ -607,11 +617,33 @@ sub validate_buddy {
         return 1 if ! wantarray;
         $self->add_error(\@errors, $field, $type, $field_val, $ifs_match);
       }
+      $content_checked = 1;
+    }
+  }
+
+  ### allow for the data to be "untainted"
+  ### this is only allowable if the user ran some other check for the datatype
+  foreach my $type ($self->filter_type('untaint',$types)) {
+    last if $#errors != -1;
+    if (! $content_checked) {
+      $self->add_error(\@errors, $field, $type, $field_val, $ifs_match);
+    } else {
+      ### generic untainter - assuming the other required content_checks did good validation
+      $_ = /(.*)/ ? $1 : die "Couldn't match?" foreach @$values;
+      if ($n_values == 1) {
+        $form->{$field} = $values->[0];
+        $self->{cgi_object}->param(-name => $field, -value => $values->[0])
+          if $self->{cgi_object};
+      } else {
+        ### values in @{ $form->{$field} } were modified directly
+        $self->{cgi_object}->param(-name => $field, -value => $values)
+          if $self->{cgi_object};
+      }
     }
   }
 
   ### all done - time to return
-  return wantarray ? @errors : scalar @errors;
+  return wantarray ? @errors : $#errors + 1;
 }
 
 ### simple error adder abstraction
@@ -631,7 +663,7 @@ sub filter_type {
   foreach (@$order) {
     push @array, $_ if /^\Q$type\E_?\d*$/;
   }
-  return wantarray ? @array : scalar @array;
+  return wantarray ? @array : $#array + 1;
 }
 
 ###----------------------------------------------------------------###
@@ -760,7 +792,7 @@ sub get_validation_keys {
         next if ! ref($field_val) && $field_val eq 'OR';
         die "Field_val must be a hashref" if ! UNIVERSAL::isa($field_val, 'HASH');
         my $key = $field_val->{'field'} || die "Missing field key in field_val hashref";
-        $keys{$key} = 1;
+        $keys{$key} = $field_val->{'name'} || 1;
       }
     } elsif ($group_val->{"group order"}) {
       die "Group order must be an arrayref" if ! UNIVERSAL::isa($group_val->{"group order"}, 'ARRAY');
@@ -769,14 +801,16 @@ sub get_validation_keys {
         next if ! $field_val && $key eq 'OR';
         die "Field_val for $key must be a hashref" if ! UNIVERSAL::isa($field_val, 'HASH');
         $key = $field_val->{'field'} if $field_val->{'field'};
-        $keys{$key} = 1;
+        $keys{$key} = $field_val->{'name'} || 1;
       }
     }
 
     ### get all others
     foreach my $key (keys %$group_val) {
       next if $key =~ /^(general|group)\s/;
-      $keys{$key} = 1;
+      my $field_val = $group_val->{$key};
+      next if ! UNIVERSAL::isa($field_val, 'HASH');
+      $keys{$key} = $field_val->{'name'} || 1;
     }
   }
 
@@ -1061,6 +1095,9 @@ sub get_error_text {
       my $_type = $field_val->{"type${dig}"};
       $return = "$name did not match type $_type.";
 
+    } elsif ($type eq 'untaint') {
+      $return = "$name cannot be untainted without one of the following checks: enum, equals, match, compare, sql, type, custom";
+
     } elsif ($type eq 'no_extra_fields') {
       $return = "$name should not be passed to validate.";
     }
@@ -1082,7 +1119,7 @@ __END__
 
 CGI::Ex::Validate - Yet another form validator - does good javascript too
 
-$Id: Validate.pm,v 1.74 2004/11/12 19:58:57 pauls Exp $
+$Id: Validate.pm,v 1.79 2005/02/23 21:28:11 pauls Exp $
 
 =head1 SYNOPSIS
 
@@ -1202,6 +1239,8 @@ return the keys of groups that were validated.
 
   my $key_hashref = $self->get_validation_keys($val_hash);
 
+The values of the hash are the names of the fields.
+
 =item C<validate>
 
 Arguments are a form hashref or cgi object, a validation hashref or filename, and
@@ -1487,11 +1526,15 @@ value may be passed of the options joined with ||.
 
 =item C<equals>
 
-Allows for comparison of two form elements.
+Allows for comparison of two form elements.  Can have an optional !.
 
   {
     field  => 'password',
     equals => 'password_verify',
+  },
+  {
+    field  => 'domain1',
+    equals => '!domain2', # make sure the fields are not the same
   }
 
 =item C<min_len and max_len>
@@ -1699,6 +1742,14 @@ would require some odd syntax for both the conditional and the default).
 
 Do what they say they do.
 
+=item C<untaint>
+
+Requires that the validated field has been also checked with
+an enum, equals, match, compare, custom, or type check.  If the
+field has been checked and there are no errors - the field is "untainted."
+
+This is for use in conjunction with the -T switch.
+
 =back
 
 =head1 ERROR OBJECT
